@@ -1,12 +1,20 @@
+import inspect
+
+
 class JsonProperty(object):
 
-    def __init__(self, default=None, name=None, choices=None):
+    default = None
+
+    def __init__(self, default=Ellipsis, name=None, choices=None, required=False):
         self.name = name
-        if not callable(default):
-            self.default = lambda: default
-        else:
+        if default is Ellipsis:
+            default = self.default
+        if callable(default):
             self.default = default
+        else:
+            self.default = lambda: default
         self.choices = choices
+        self.required = required
 
     def init_property(self, default_name):
         self.name = self.name or default_name
@@ -47,10 +55,40 @@ class JsonProperty(object):
         self.name = self.name or method.func_name
         return self
 
+    def empty(self, value):
+        return value is None
+
     def validate(self, value):
-        if self.choices and value not in self.choices:
-            raise ValueError('{0!r} not in choices: {1!r}'.format(
-                value, self.choices))
+        if self.choices and value not in self.choices and value is not None:
+            raise ValueError(
+                '{0!r} not in choices: {1!r}'.format(value, self.choices)
+            )
+        if self.empty(value) and self.required:
+            raise ValueError(
+                'Required property received value {0!r}'.format(value)
+            )
+
+
+class JsonContainerProperty(JsonProperty):
+
+    def __init__(self, obj_type=None, **kwargs):
+        self._obj_type = obj_type
+        super(JsonContainerProperty, self).__init__(**kwargs)
+
+    @property
+    def obj_type(self):
+        if inspect.isfunction(self._obj_type):
+            return self._obj_type()
+        else:
+            return self._obj_type
+
+
+class DefaultProperty(JsonProperty):
+    def wrap(self, obj):
+        return obj
+
+    def unwrap(self, obj):
+        return obj, obj
 
 
 class AssertTypeProperty(JsonProperty):
@@ -69,10 +107,20 @@ class AssertTypeProperty(JsonProperty):
         return obj, obj
 
 
+def check_type(obj, obj_type, message):
+    if obj is None:
+        return obj_type()
+    elif not isinstance(obj, obj_type):
+        raise ValueError(message)
+    else:
+        return obj
+
+
 class JsonArray(list):
-    def __init__(self, obj, wrapper=None):
+    def __init__(self, _obj=None, wrapper=None):
         super(JsonArray, self).__init__()
-        self._obj = obj
+
+        self._obj = check_type(_obj, list, 'JsonArray must wrap a list or None')
         self._wrapper = wrapper
         for item in self._obj:
             super(JsonArray, self).append(self._wrapper.wrap(item))
@@ -96,6 +144,34 @@ class JsonArray(list):
         super(JsonArray, self).extend(wrapped_list)
 
 
+class JsonDict(dict):
+
+    def __init__(self, _obj=None, wrapper=None, **kwargs):
+        super(JsonDict, self).__init__()
+        self._obj = check_type(_obj, dict, 'JsonDict must wrap a dict or None')
+        self._wrapper = wrapper or DefaultProperty()
+
+        for key, value in self._obj.items():
+            self[key] = self.__wrap(key, value)
+
+    def __wrap(self, key, unwrapped):
+        return self._wrapper.wrap(unwrapped)
+
+    def __unwrap(self, key, wrapped):
+        return self._wrapper.unwrap(wrapped)
+
+    def __setitem__(self, key, value):
+        wrapped, unwrapped = self.__unwrap(key, value)
+        self._obj[key] = unwrapped
+        super(JsonDict, self).__setitem__(key, wrapped)
+
+    def update(self, E=None, **F):
+        for dct in (E, F):
+            if dct:
+                for key, value in dct.items():
+                    self[key] = value
+
+
 class JsonObjectMeta(type):
     # There's a pretty fundamental cyclic dependency between this metaclass
     # and knowledge of all available property types (in properties module).
@@ -110,7 +186,7 @@ class JsonObjectMeta(type):
         for key, value in dct.items():
             if isinstance(value, JsonProperty):
                 properties[key] = value
-            elif key == '__module__':
+            elif key in ('__module__', '__doc__'):
                 continue
             elif _p and isinstance(value, tuple(_p.TYPE_TO_PROPERTY.keys())):
                 property_ = _p.type_to_property(type(value), default=value)
@@ -143,11 +219,14 @@ class JsonObject(dict):
 
     _properties_by_attr = None
     _properties_by_key = None
+    __save_dynamic_properties = False
 
     def __init__(self, _obj=None, **kwargs):
         super(JsonObject, self).__init__()
 
-        self._obj = _obj or {}
+        self._obj = check_type(_obj, dict,
+                               'JsonObject must wrap a dict or None')
+        self.__save_dynamic_properties = True
 
         for key, value in self._obj.items():
             self[key] = self.__wrap(key, value)
@@ -172,20 +251,35 @@ class JsonObject(dict):
     def to_json(self):
         return self._obj
 
+    def __get_property(self, key):
+        try:
+            return self._properties_by_key[key]
+        except KeyError:
+            return DefaultProperty()
+
     def __wrap(self, key, value):
+        property_ = self.__get_property(key)
+
         if value is None:
             return None
-        return self._properties_by_key[key].wrap(value)
+
+        return property_.wrap(value)
 
     def __unwrap(self, key, value):
+        property_ = self.__get_property(key)
+        property_.validate(value)
+
         if value is None:
             return None, None
 
-        property_ = self._properties_by_key[key]
-        property_.validate(value)
         return property_.unwrap(value)
 
     def __setitem__(self, key, value):
         wrapped, unwrapped = self.__unwrap(key, value)
         super(JsonObject, self).__setitem__(key, wrapped)
         self._obj[key] = unwrapped
+
+    def __setattr__(self, name, value):
+        if self.__save_dynamic_properties and name not in self._properties_by_attr:
+            self[name] = value
+        super(JsonObject, self).__setattr__(name, value)
